﻿<#
Script      : url_extractor.ps1
Description : Extracts all URLs from a text based file (e.g. .txt, .xml, .html)  
              and saves them in the desired format as a list to a new textfile
Requirements: PowerShell 3.0 or later
              This script file has to stay in UTF-8-BOM encoding to show umlauts correctly
Author      : Bjoern Bastian, aufschnur.de
Version     : 2017-03-13
#>

# CONFIGURATION ----------------------------------------------------------------

# Before you edit this script be sure that you are using a text editor that can 
# handle and save the document in UTF-8-BOM encoding! Like the free Notepad++ editor.

# Text based input file (txt, html, xml, ...) to extract URLs from
[String]$input_file = "input.txt"

# Output file were the extracted URLs will be saved in
[String]$output_file = "extracted-urls.txt"

# ------------------------------------------------------------------------------

# DO NOT TOUCH ANYTHING BELOW HERE UNLESS YOU KNOW WHAT YOU ARE DOING

Clear-Host
# Set script language
$langid = 0;
# English by default, switch to german if german OS detected
If ($PSUICulture.StartsWith("de")) { $langid = 1}
# Hashtable-array-construct for the language strings
$langstrings = @{
	PressKey = @(
		"Press any key to exit...`n", 
		"Drücken Sie eine beliebige Taste zum Beenden...`n"	)
	Err_PS3 = @(
		"Error: PowerShell version 3 or newer is required for this script.`nIt seems that there is an older version installed on this system.`n",
		"Fehler: Dieses Skript benötigt PowerShell Version 3 oder neuer.`nEs scheint eine ältere Version auf diesem System installiert zu sein.`n" )
	Err_Inputfile = @(
		"Error: Input file '{0}' not found!`n",
		"Fehler: Quelldatei '{0}' wurde nicht gefunden!`n" )
	Err_Outputfile = @(
		"`nError: Output file could not be created!`nPossibly no write access for this directory or the file is currently open by another program.`n",
		"`nFehler: Die Ergebnisdatei konnte nicht erstellt werden!`nEventuell keine Schreibberechtigung für dieses Verzeichnis, oder die Datei ist momentan durch ein anderes Programm geöffnet.`n" )
	Inputfile = @(
		"Read in input file...`n",
		"Lese Quelldatei ein...`n" )
	Extract = @(
		"Extract URLs...",
		"Extrahiere URLs..." )
	FileInfo = @(
		"Source file: {0}`nFile size  : {1} KB`n", 
		"Quelldatei: {0}`nDateigröße: {1} KB`n" )
	UniquesFound = @(
		"Unique URLs found:", 
		"Eindeutige URLs gefunden:" )
	CleanUp = @(
		"Clean up URLs...`n",
		"Säubere URLs...`n" )
	FinalInfo = @(
		"DONE! --> {0} unique URLs found!`nResults were saved in the file '{1}'.`n", 
		"FERTIG! --> {0} eindeutige URLs gefunden!`nDie Ergebnisse wurden in die Datei '{1}' gespeichert.`n" )
	Info = @( @"
NOTE: This script extracts URLs from any text based files (like .html, .xml,
      .txt). This script is NOT a parser! It does not analyze any HTML or XML 
      tags but only recognize URLs with `"http://`", `"https://`", `"ftp://`" 
      etc. in it.`n`n
"@, @"
HINWEIS: Dieses Skript extrahiert URLs aus textbasierten Dateien (z.B. .html,
         .xml, .txt). Dieses Skript ist KEIN Parser! Es analysiert keine HTML- 
         oder XML-Tags sondern erkennt lediglich URLs mit `"http://`", 
         `"https://`", `"ftp://`" etc.`n`n
"@)
	Mode = @( @"
In which format should the URLs be stored? 
Press the number of the mode you would like to use.`n
( 1 ) Complete URL string including path, query etc.
      e.g. http://www.example.com/subdir/index.html?p1=A#ressource`n
( 2 ) URL with protocol and host/(sub)domains
      e.g. http://www.example.com`n
( 3 ) URI (not URL) with host/(sub)domains only (except www.)
      e.g. example.com`n
Your choice: 
"@,	@"
In welchem Format sollen die URLs gespeichert werden? 
Drücken Sie die Zahl des gewünschten Modus.`n
( 1 ) Komplette URL inklusive ggfs. Pfad, Query, usw.
      z.B. http://www.example.com/subdir/index.html?p1=A#ressource`n
( 2 ) URL mit Protokoll und Host/(Sub)Domains
      z.B. http://www.example.com`n
( 3 ) URI (nicht URL) mit lediglich Host/(Sub)Domains (außer www.)
      z.B. example.com`n
Ihre Wahl: 
"@ )
}
	
function End-Script {
	Write-Host -NoNewLine $langstrings.PressKey[$langid]
	$Host.UI.RawUI.ReadKey("NoEcho,IncludeKeyDown") | Out-Null
	Write-Host ""
	Exit
}
$Host.UI.RawUI.WindowTitle = "URL Extractor"
Write-Host "`n-------------`nURL EXTRACTOR`n-------------`n"
# Check for PowerShell version 3 or newer
[int]$psversion = [convert]::ToInt32(((Get-Host).Version).Major, 10)
If ($psversion -lt 3) { Write-Host $langstrings.Err_PS3[$langid] -Foreground "Red"; End-Script }
# Some info about the script
Write-Host -NoNewLine $langstrings.Info[$langid] -Foreground "Gray"
# Get file content
Write-Host $langstrings.Inputfile[$langid]
If (!(Test-Path $input_file)) { Write-Host ($langstrings.Err_Inputfile[$langid] -f $input_file) -Foreground "Red"; End-Script }
# Some info about the input file
$content_size = '{0:N0}' -f ((Get-Item -Path $input_file).length / 1KB)
Write-Host ($langstrings.FileInfo[$langid] -f $input_file, $content_size) -Foreground "Gray"
"=============" + [System.Environment]::NewLine + "URL EXTRACTOR" + [System.Environment]::NewLine + "=============" + [System.Environment]::NewLine + [System.Environment]::NewLine + $langstrings.UniquesFound[$langid] + [System.Environment]::NewLine > $matchfile

# Ask for storage modus
Write-Host -NoNewLine $langstrings.Mode[$langid]
Do {
	$key = [Console]::ReadKey($True)
}
While (($key.KeyChar -ne "1") -And ($key.KeyChar -ne "2") -And ($key.KeyChar -ne "3"))
Write-Host $key.KeyChar "`n"

# Lets extract the URLs...
Write-Host $langstrings.Extract[$langid]
[System.Collections.ArrayList]$arr_urls_raw = @()
[System.Collections.ArrayList]$arr_urls_final = @()
$regex = '([a-zA-Z]{3,})://([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?'
# Find all unique URLs and write them into output file
try 
{
	Select-String -Path $input_file -Pattern $regex -AllMatches | % { $_.Matches } | % { $_.Value } | Sort | Get-Unique > $output_file
} catch {
	Write-Host $langstrings.Err_Outputfile[$langid] -Foreground "Red"; End-Script
}
Write-Host $langstrings.CleanUp[$langid]
# Read in complete list
$arr_urls_raw = Get-Content -Path $output_file
# Check every entry
$arr_urls_raw | Foreach-Object {
	$domain = $_; 
	Switch ($key.KeyChar)
	{
		"1" {
			# Full URL string: Don't cut anything
			# Just for the sake of completeness
		}
		"2" {
			# URL: Keep protocol and host including all subdomains
			$domain = (($domain).TrimEnd("/")) -Replace '^([a-zA-Z]{3,}://[^/]+).*', '$1'
		}
		"3" {
			# URI: Keep only host and (sub)domains (except www.)
			$domain = ([System.Uri]$domain).Host -Replace '^www\.'
		}
	}; 
	[void]$arr_urls_final.Add($domain)
}
# Save list to file
$arr_urls_final | Sort | Get-Unique | Set-Content -Path $output_file 
# Give info
$urls_count = ($arr_urls_final | Sort | Get-Unique).Count
Write-Host ($langstrings.FinalInfo[$langid] -f $urls_count, $output_file) -Foreground "Green"
# Release the krak... memory
[void]$arr_urls_raw.Clear()
[void]$arr_urls_final.Clear()
[System.GC]::Collect()
End-Script
